@@ -10,7 +10,7 @@ use Carp::Clan qw/^DBIx::Class/;
 use namespace::clean;
 use DBIx::Class::Schema::Loader::Table ();
 
-our $VERSION = '0.07036';
+our $VERSION = '0.07037';
 
 __PACKAGE__->mk_group_accessors('simple', qw/
     _disable_pk_detection
@@ -61,7 +61,7 @@ sub new {
         }
     }
 
-    # Set up the default quoting character and name seperators
+    # Set up the default quoting character and name separators
     $self->quote_char($self->_build_quote_char);
     $self->name_sep($self->_build_name_sep);
 
@@ -194,6 +194,40 @@ sub _tables_list {
     return $self->_filter_tables(\@tables, $opts);
 }
 
+sub _recurse_constraint {
+    my ($constraint, @parts) = @_;
+
+    my $name = shift @parts;
+
+    # If there are any parts left, the constraint must be an arrayref
+    croak "depth of constraint/exclude array does not match length of moniker_parts"
+        unless !!@parts == !!(ref $constraint eq 'ARRAY');
+
+    # if ths is the last part, use the constraint directly
+    return $name =~ $constraint unless @parts;
+
+    # recurse into the first matching subconstraint
+    foreach (@{$constraint}) {
+        my ($re, $sub) = @{$_};
+        return _recurse_constraint($sub, @parts)
+            if $name =~ $re;
+    }
+    return 0;
+}
+
+sub _check_constraint {
+    my ($include, $constraint, @tables) = @_;
+
+    return @tables unless defined $constraint;
+
+    return grep { !$include xor _recurse_constraint($constraint, @{$_}) } @tables
+        if ref $constraint eq 'ARRAY';
+
+    return grep { !$include xor /$constraint/ } @tables;
+}
+
+
+
 # apply constraint/exclude and ignore bad tables and views
 sub _filter_tables {
     my ($self, $tables, $opts) = @_;
@@ -202,11 +236,8 @@ sub _filter_tables {
     my @filtered_tables;
 
     $opts ||= {};
-    my $constraint   = $opts->{constraint};
-    my $exclude      = $opts->{exclude};
-
-    @tables = grep { /$constraint/ } @tables if defined $constraint;
-    @tables = grep { ! /$exclude/  } @tables if defined $exclude;
+    @tables = _check_constraint(1, $opts->{constraint}, @tables);
+    @tables = _check_constraint(0, $opts->{exclude}, @tables);
 
     TABLE: for my $table (@tables) {
         try {
@@ -341,10 +372,9 @@ WHERE table_name = @{[ $dbh->quote($table->name) ]}
 EOF
 
     # Failback: try the REMARKS column on table_info
-    if (!$comment && $dbh->can('table_info')) {
-        my $sth = $self->_dbh_table_info( $dbh, undef, $table->schema, $table->name );
-        my $info = $sth->fetchrow_hashref();
-        $comment = $info->{REMARKS};
+    if (!$comment) {
+        my $info = $self->_dbh_table_info( $dbh, $table );
+        $comment = $info->{REMARKS} if $info;
     }
 
     return $comment;
@@ -601,9 +631,23 @@ sub _extra_column_info {}
 
 # override to mask warnings if needed
 sub _dbh_table_info {
-    my ($self, $dbh) = (shift, shift);
+    my ($self, $dbh, $table) = (shift, shift, shift);
 
-    return $dbh->table_info(@_);
+    return undef if !$dbh->can('table_info');
+    my $sth = $dbh->table_info(undef, $table->schema, $table->name);
+    while (my $info = $sth->fetchrow_hashref) {
+        next if !$self->_table_info_matches($table, $info);
+        return $info;
+    }
+    return undef;
+}
+
+sub _table_info_matches {
+    my ($self, $table, $info) = @_;
+
+    no warnings 'uninitialized';
+    return $info->{TABLE_SCHEM} eq $table->schema
+        && $info->{TABLE_NAME}  eq $table->name;
 }
 
 # override to mask warnings if needed (see mysql)
@@ -636,6 +680,14 @@ sub dbh {
     return $self->schema->storage->dbh;
 }
 
+sub _table_is_view {
+    my ($self, $table) = @_;
+
+    my $info = $self->_dbh_table_info($self->dbh, $table)
+        or return 0;
+    return $info->{TABLE_TYPE} eq 'VIEW';
+}
+
 =head1 SEE ALSO
 
 L<DBIx::Class::Schema::Loader>
